<?php
/**
 * Plugin Name: WP Ultimo - PRO Theme
 * Plugin URI: https://wpultimo.com/addons
 * Text Domain: pro-theme
 * Description: Give a professional look to your WP Admin interface.
 * Version: 1.1.5
 * Author: Arindo Duque - NextPress
 * Author URI: http://nextpress.co/
 * Copyright: Arindo Duque, NextPress
 * Network: true
 * License: GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 *
 * PRO Theme is distributed under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * PRO Theme is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PRO Theme. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author   Arindo Duque and NextPress
 * @category Add-on
 * @package  WP_Ultimo/Addons
 * @version  1.1.0
 */

require_once plugin_dir_path(__FILE__).'/paradox/paradox.php';

require_once plugin_dir_path(__FILE__).'/inc/np-theme-factory.php';

if (!class_exists('PROTheme')) :

/**
 * Here starts our plugin.
 */
class PROTheme extends NextPress_Theme_Factory  {

  /*
   * Remove the WordPress Logo from the WordPress Admin Bar
   */
  public function editAdminBar() {
    
    // If it does not go to the backend, does even show
    if (!is_admin() && !$this->options->getOption('admin-bar-frontend')) return;
    
    // Get global var admin bar
    global $wp_admin_bar;
    
    // Remove undesired nodes
    // $wp_admin_bar->remove_menu('my-account');
    $wp_admin_bar->remove_menu('wp-logo');
    
    
    $myAccount = $wp_admin_bar->get_node('my-account');
    $newTitle = str_replace('Howdy,', '<span class="hidden-xs hidden-sm hidden-md">'.__('Welcome back,', 'pro-theme').'</span><strong>', $myAccount->title);
    $newTitle = str_replace('s=26', 's=40', $newTitle);
    $newTitle = str_replace('<img', '</strong><img', $newTitle);
    
    // var_dump($newTitle);
    
    $wp_admin_bar->add_node(array(
      'id' => 'my-account',
      'title' => $newTitle,
    ));
  }

    /**
     * Call private functions/actions in children class;
     * private_theme_functions() called into NextPress_Theme_Factory::Plugin()
     *  
     * @return void
     */
    public function private_theme_functions() {
      // Clean Footer on the Right
      add_filter('update_footer', '__return_false', 99999);
    }

    /**
   * Add a class or many to the body in the dashboard
   * @param  string $classes Classes already attached to the admin body class
   * @return string New body classes string
   */
  public function addControlAdminClasses($classes) {
    
    // Our control classes carrier
    $controlClasses = array();
    
    // If the menu position is switched, add the control class "pro-theme-menu-$position"
    $controlClasses[] = "pro-theme-menu-".$this->options->getOption('menu-position');
    
    // Add class for opacity if the user have selected
    if ($this->options->getOption('parallax-options') == 'parallax') {
      $controlClasses[] = 'pro-theme-no-opacity';
    }
    // Arrows or Icons
    // Added in 0.0.5
    if ($this->options->getOption('menu-icons') == true) {
      $controlClasses[] = 'pro-theme-menu-arrows';
    } // end if;
    if ($this->options->getOption('menu-icons-display') == true) {
      
      $controlClasses[] = 'pro-theme-menu-icons';
      
    } // end if;
    // Add classes related to the admin menu
    // Added in 0.0.5
    $controlClasses[] =$this->options->getOption('menu-arrows-all') == true
      ? 'pro-theme-menu-arrows-all'
      : 'pro-theme-menu-arrows-only';
    
    // Return the classes plus our own
    return "$classes ".implode(' ', $controlClasses);
    
  } // end addControlAdminClasses;

  
} // end PROTheme;

function initialize_pro_theme() {
  
    if (class_exists('NextPress_Theme_Factory')) {
        // Now we need to load our config file
        $config = include plugin_dir_path(__FILE__).'/config.php';

        /**
        * We execute our plugin, passing our config file
        */
        $PROTheme = new PROTheme($config);
        $PROTheme->_refer_js = 'commonL10n';
    }
}

if (!defined('PRO_THEME_IS_NETWORK')) {

  if (!function_exists('is_plugin_active_for_network')) {
    require_once(ABSPATH . '/wp-admin/includes/plugin.php');
  } // end if;

  define('PRO_THEME_IS_NETWORK', is_plugin_active_for_network(plugin_basename(__FILE__)));

} // end if;

require_once plugin_dir_path(__FILE__) . 'init.php';

endif;

